/*
    Copyright (C) 2003 Fons Adriaensen
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

//-----------------------------------------------------------------------------------
// Common definitions


#include "moogvcf.h"

#define IDENT 1941
#define NMODS 3
#define VERSION "1.0"


static const char* maker = "Fons Adriaensen <fons.adriaensen@alcatel.be>";
static const char* copyr = "(C) 2003 Fons Adriaensen, GNU General Public License Version 2 applies";


static void pconnect (LADSPA_Handle H, unsigned long port, LADSPA_Data *data)
{
  ((LadspaPlugin *)H)->setport (port, data);
}

static void activate (LADSPA_Handle H)
{
  ((LadspaPlugin *)H)->active (true);
}

static void runplugin (LADSPA_Handle H, unsigned long k)
{
  ((LadspaPlugin *)H)->runproc (k, false);
}

static void runadding (LADSPA_Handle H, unsigned long k)
{
  ((LadspaPlugin *)H)->runproc (k, true);
}

static void setadding (LADSPA_Handle H, LADSPA_Data gain)
{
  ((LadspaPlugin *)H)->setgain (gain);
}

static void deactivate (LADSPA_Handle H)
{
  ((LadspaPlugin *)H)->active (false);
}

static void cleanup (LADSPA_Handle H)
{
  delete (LadspaPlugin *) H;
}

//-----------------------------------------------------------------------------------
// Plugin definitions


static const char* label0 = "MoogVCF1";
static const char* name0  = "MoogVCF1";

static LADSPA_Handle instant0 (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
  return new Ladspa_Moogvcf1 (rate);
}

static const char* label1 = "MoogVCF2";
static const char* name1  = "MoogVCF2";

static LADSPA_Handle instant1 (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
  return new Ladspa_Moogvcf2 (rate);
}

static const char* label2 = "MoogVCF3";
static const char* name2  = "MoogVCF3";

static LADSPA_Handle instant2 (const struct _LADSPA_Descriptor *desc, unsigned long rate)
{
  return new Ladspa_Moogvcf3 (rate);
}

static const LADSPA_PortDescriptor pdesc0 [Ladspa_Moogvcf1::NPORT] = 
{
  LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
  LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO,
  LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
  LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
  LADSPA_PORT_INPUT  | LADSPA_PORT_AUDIO,
  LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL,
  LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL,
  LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL,
  LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL,
  LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL,
  LADSPA_PORT_INPUT  | LADSPA_PORT_CONTROL
};

static const char * const pname0  [Ladspa_Moogvcf1::NPORT] = 
{
  "Input",
  "Output",
  "Frequency",
  "Exp FM",
  "Resonance",
  "Input gain (dB)",
  "Frequency",
  "Exp FM gain",
  "Resonance",
  "Resonance gain",
  "Output gain (dB)"
};

static const LADSPA_PortRangeHint phint0 [Ladspa_Moogvcf1::NPORT] = 
{
  { 0, 0, 0 },
  { 0, 0, 0 },
  { 0, 0, 0 },
  { 0, 0, 0 },
  { 0, 0, 0 },
  { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0, -60, 10 },
  { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0, -5, 5 },
  { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, 0, 10 },
  { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, 0, 1 },
  { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE, 0, 1 },
  { LADSPA_HINT_BOUNDED_BELOW | LADSPA_HINT_BOUNDED_ABOVE | LADSPA_HINT_DEFAULT_0, -15, 15 },
};


static const LADSPA_Descriptor moddescr [NMODS] =
{
  {
    IDENT+0,
    label0,
    LADSPA_PROPERTY_REALTIME | LADSPA_PROPERTY_HARD_RT_CAPABLE,
    name0,
    maker,
    copyr,
    Ladspa_Moogvcf1::NPORT,
    pdesc0,
    pname0,
    phint0,
    0,
    instant0,
    pconnect,
    activate,
    runplugin,
    runadding,
    setadding,
    deactivate,
    cleanup
  },  
  {
    IDENT+1,
    label1,
    LADSPA_PROPERTY_REALTIME | LADSPA_PROPERTY_HARD_RT_CAPABLE,
    name1,
    maker,
    copyr,
    Ladspa_Moogvcf2::NPORT,
    pdesc0,
    pname0,
    phint0,
    0,
    instant1,
    pconnect,
    activate,
    runplugin,
    runadding,
    setadding,
    deactivate,
    cleanup
  },  
  {
    IDENT+2,
    label2,
    LADSPA_PROPERTY_REALTIME | LADSPA_PROPERTY_HARD_RT_CAPABLE,
    name2,
    maker,
    copyr,
    Ladspa_Moogvcf3::NPORT,
    pdesc0,
    pname0,
    phint0,
    0,
    instant2,
    pconnect,
    activate,
    runplugin,
    runadding,
    setadding,
    deactivate,
    cleanup
  }  
};

extern "C" const LADSPA_Descriptor *ladspa_descriptor (unsigned long i)
{
  if (i >= NMODS) return 0;
  return moddescr + i;
}

//-----------------------------------------------------------------------------------
